# Copyright The Numerical Algorithms Group Limited 1992-94.
#
# This awk script processes an .htex file and creates a spad
# .input file on standard output. Two arguments are expected.
# The first is the name of the htex file without the .htex suffix
# and the second is the  directory (with trailing /) where the
# the spad spool file goes.

BEGIN           {
        fn = ARGV[1]
        dir= ARGV[2]
        ARGV[1] = fn".htex"
        ARGC = 2
        print  "-- % Copyright The Numerical Algorithms Group Limited 1992-94. All rights reserved."
        printf "-- %%%% Generated by input.awk from %s\n", ARGV[1]
        print ")lib )dir ."
	print ")startNAGLink"
	print ")startGraphics"
	print ")set nag host boston"
        print ")set user devel"
        print ")set fortran defaulttype REAL"
        print ")set fortran fortindent  6"
        print ")set fortran fortlength  72"
        print ")set fortran ints2floats on"
        print ")set fortran optlevel    0"
        print ")set fortran precision   double"
        print ")set fortran segment     on"
        print ")set fortran startindex  1"
        print ")set fortran typedecs on"
        print ")set message autoload off"
        print ")set message bottomup  off"
        print ")set message highlight off"
        print ")set message time off"
        print ")set message type on"
        print ")set out algebra console"
        print ")set out algebra off"
        print ")set out char plain"
        print ")set out fortran console"
        print ")set out fortran off"
        print ")set out length 62"
        print ")set out script console"
        print ")set out script off"
        print ")set out tex console"
        print ")set out tex on"
        print ")set streams calculate 7"
        print ")trace )off"
        print ")clear all"
        printf ")system rm -f %s%s.out\n", dir, fn
        printf ")spool %s%s.out\n", dir, fn
}

/^\\begin{spadsrc}/ {
        printf "-- %s\n",$0
        i = 0
        getline
        while ($1 != "\\end{spadsrc}") {
          gsub(/\\/,"")
          ss[++i] = $0
          getline
        }
        printf "-- %d\n",i
        for (j = 1; j <= i; j++) {
          print ss[j]
          delete ss[j]
        }
        next
}

/^\\begin{inputonly}/ {
        printf "-- %s\n",$0
        getline
        while (substr($1,1,15) != "\\end{inputonly}") {
                print $0
                getline
        }
        print "-- \\end{inputonly}"
        next
}

/^\\spadgraph/ {
        # fix special characters
        gsub(/\\ /," ")
        gsub(/\\_/,"_")
        gsub(/\\}/,"}")
        gsub(/\\{/,"{")
        gsub(/\\#/,"#")
        gsub(/\\%/,"%")
        gsub(/\\\$/,"\$")
# gsub with & just does not seem to work, even with \& in substitution string
#       gsub(/\\&/,"\&")
        pa = index($0,"\\&")
        while (pa > 0) {
          $0 = substr($0,1,pa-1) "&" substr($0,pa+2)
          pa = index($0,"\\&")
        }
        # get rid of prefix and suffix
        $0 = substr($0,12)     # prefix
        # remove trailing blanks and last brace
        while (substr($0,length($0),1) == " ")
                $0 = substr($0,1,length($0)-1)
        if (substr($0,length($0),1) == "}")
                $0 = substr($0,1,length($0)-1)
        split($0,x,"\\") # get rid of \free or \bound
        print x[1]
        next
}

/^\\spadcommand/ {
        # fix special characters
        gsub(/\\ /," ")
        gsub(/\\_/,"_")
        gsub(/\\}/,"}")
        gsub(/\\{/,"{")
        gsub(/\\#/,"#")
        gsub(/\\%/,"%")
        gsub(/\\\$/,"\$")
# gsub with & just does not seem to work, even with \& in substitution string
#       gsub(/\\&/,"\&")
        pa = index($0,"\\&")
        while (pa > 0) {
          $0 = substr($0,1,pa-1) "&" substr($0,pa+2)
          pa = index($0,"\\&")
        }
        # get rid of prefix and suffix
        $0 = substr($0,14)     # prefix
        # remove trailing blanks and last brace
        while (substr($0,length($0),1) == " ")
                $0 = substr($0,1,length($0)-1)
        if (substr($0,length($0),1) == "}")
                $0 = substr($0,1,length($0)-1)
        split($0,x,"\\") # get rid of \free or \bound
        print x[1]
        next
}

# otherwise
                {
        printf "-- %s\n",$0
        if (match($0,/^\\head/))
          print ")clear all"
        next
}

END             {
        print ")spool"
        print ")set fortran defaulttype REAL"
        print ")set fortran fortindent  6"
        print ")set fortran fortlength  72"
        print ")set fortran ints2floats on"
        print ")set fortran optlevel    0"
        print ")set fortran precision   double"
        print ")set fortran segment     on"
        print ")set fortran startindex  1"
        print ")set fortran typedecs on"
        print ")set out algebra console"
        print ")set out algebra on"
        print ")set out fortran console"
        print ")set out fortran off"
        print ")set out length 76"
        print ")set out script console"
        print ")set out script off"
        print ")set out tex console"
        print ")set out tex off"
        print ")set streams calculate 10"
	print ")stopNAGLink"
	print ")stopGraphics"
}
