/*
Copyright (c) 1991-2002, The Numerical ALgorithms Group Ltd.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    - Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.

    - Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the
      distribution.

    - Neither the name of The Numerical ALgorithms Group Ltd. nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/***********************************************************************
 * mkhh.c
 *
 * Copyright Numerical Algorithms Group, Ltd. 1992-94. All rights reserved.
 *
 * Sample compilation command:  cc -O -o mkhh mkhh.c
 *
 * Sample usage: mkhh /spad/int/doc/htex/pages/ [A-Z]*.htex > /tmp/xmps.mak
 *               make -f /tmp/xmps.mak
 *
 * This program accepts as command line arguments a list of .htex files
 * and compares their dates to the corresponding .ht files in the
 * mount directory. A makefile is created on standard output that
 * contains the statements needed to make the necessary files.
 * The first command line argument, which is required, is the directory
 * in which the generated .ht files should be placed.
 *
 * It is assumed that the AXIOM variable is set correctly and that the
 * files are in the current directory and are specified without the
 * directory name.
 *
 * Robert S. Sutor       06/92
 **********************************************************************/
#include "useproto.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#define FILENAMEBUFFERSIZE 513

int
#ifndef _NO_PROTO
main(int argc, char **argv)
#else
main(argc,argv)
int argc; 
char **argv;
#endif
{

  struct FileData {
    char  *name;
    time_t htex_time;
    time_t ht_time;
    int    newer;
  } *fileData;

  struct stat fileStats;

  int i, j, fileCount, notexists = 0, newer = 0;

  char *filePathBuffer, *htPath;

  if (argc < 2) {
    printf("Error (mkhh): first argument (a directory with trailing '/' is required.\n");
    exit(1);
  }
  htPath = argv[1];


  fileCount = argc - 2;
  fileData = (struct FileData *) malloc(fileCount * sizeof(struct FileData));
  filePathBuffer = (char *) malloc(FILENAMEBUFFERSIZE);

  for (i = 0; i < argc-2; i++) {
    fileData[i].name = argv[i+2];
    fileData[i].newer = 0;

    /* get file stats for .htex file */

    strcpy(filePathBuffer,argv[i+2]);

    if (! stat(filePathBuffer, &fileStats))
      fileData[i].htex_time = fileStats.st_mtime;
    else {
      /* file does not exist */

      fileData[i].htex_time = fileData[i].ht_time = 0;
      continue;
    }

    /* get file stats for .ht file */

    strcpy(filePathBuffer,htPath);
    strcat(filePathBuffer,argv[i+2]);
    filePathBuffer[strlen(filePathBuffer)-2] = '\0'; /* lop of 'ex' */

    if (! stat(filePathBuffer, &fileStats))
      fileData[i].ht_time = fileStats.st_mtime;
    else {
      fileData[i].ht_time = 0;
      notexists++;
    }

    if (fileData[i].ht_time && (fileData[i].htex_time > fileData[i].ht_time)) {
      newer++;
      fileData[i].newer = 1;
    }
  }

  printf("# Makefile created by mkhh.\n\nall:\n");

  if (notexists)
    /* have to redo them all */
    printf("\t$(SRC)/htex/mkxmps $(HTCHAP) $(HTHTDIR) $(HTINDIR) $(HTOUTDIR) $(HTTEXDIR)\n");
  else if (newer)
    for (i = 0; i < fileCount; i++) {
      if (fileData[i].newer) {
        for (j = strlen(fileData[i].name) - 1; j > -1; j--) {
          if (fileData[i].name[j] == '.') {
            fileData[i].name[j] = '\0';
            break;
          }
        }
        printf("\t$(SRC)/htex/mkhtx %s %s\n",fileData[i].name,htPath);
      }
    }
}
