/*
Copyright (c) 1991-2002, The Numerical ALgorithms Group Ltd.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    - Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.

    - Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the
      distribution.

    - Neither the name of The Numerical ALgorithms Group Ltd. nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/***********************************************************************
 * mktinput.c
 *
 * Copyright Numerical Algorithms Group, Ltd. 1994. All rights reserved.
 *
 * Sample compilation command:  cc -O -o mktinput mktinput.c
 *
 * Sample usage: mktinput /spad/int/input/ [A-Z]*.htex > /tmp/xmps.mak
 *               make -f /tmp/xmps.mak
 *
 * This program accepts as command line arguments a list of .htex or .ht files
 * and compares their dates to the corresponding test .input files in the
 * directory which is the first argument. A makefile is created on standard
 * output that contains the statements needed to make the necessary test input
 * files.
 * The first command line argument, which is required, is the directory
 * in which the generated .ht files should be placed.
 *
 * It is assumed that the AXIOM variable is set correctly and that the
 * files are in the current directory and are specified without the
 * directory name.
 *
 * Robert S. Sutor       05/94
 **********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#define FILENAMEBUFFERSIZE 513

int
main(int argc, char **argv)
{

  struct FileData {
    char  *name;
    char  *baseName;
    time_t source_time;
    time_t input_time;
    int    newer;
  } *fileData;

  struct stat fileStats;

  int i, j, fileCount, newer = 0;

  char *filePathBuffer, *inputPath;

  if (argc < 2) {
    printf("Error (mkhh): first argument (a directory with trailing '/' is required.\n");
    exit(1);
  }
  inputPath = argv[1];


  fileCount = argc - 2;
  fileData = (struct FileData *) malloc(fileCount * sizeof(struct FileData));
  filePathBuffer = (char *) malloc(FILENAMEBUFFERSIZE);

  for (i = 0; i < argc-2; i++) {
    char *dot;
    fileData[i].name = argv[i+2];
    fileData[i].baseName = strdup(argv[i+2]);
    dot = strrchr(fileData[i].baseName, '.');
    if (dot)
        *dot = '\0';
    fileData[i].newer = 0;

    /* get file stats for .htex or .ht file  */

    strcpy(filePathBuffer,argv[i+2]);

    if (! stat(filePathBuffer, &fileStats))
      fileData[i].source_time = fileStats.st_mtime;
    else {
      /* file does not exist */

      fileData[i].source_time = fileData[i].input_time = 0;
      continue;
    }

    /* get file stats for .input file */

    strcpy(filePathBuffer,inputPath);
    strcat(filePathBuffer,fileData[i].baseName);
    strcat(filePathBuffer,".input");

    if (! stat(filePathBuffer, &fileStats)) {
      fileData[i].input_time = fileStats.st_mtime;
      if (fileData[i].source_time > fileData[i].input_time) {
        newer++;
        fileData[i].newer = 1;
      }
    }
    else {
      fileData[i].input_time = 0;
      newer++;
      fileData[i].newer = 1;
    }
  }

  printf("# Makefile created by mktinput.\n\nall:\n");

  if (newer)
    for (i = 0; i < fileCount; i++) {
      if (fileData[i].newer) {
        printf("\t@ echo Generating %s%s.input\n",inputPath, fileData[i].baseName);
        printf("\t@ ${AWK} -f ${SRC}/htex/tinput.awk %s > %s%s.input\n",fileData[i].name, inputPath, fileData[i].baseName);
      }
    }
}
