# Copyright The Numerical Algorithms Group Limited 1991-94.
#
# This program translates o<suffix>.tex files so that:
#   1) names of system functions and parameters appear in {\tt }
#   2) user-specifiable parameters appear in italics or math mode
#
# Specific transformations:
#   1)  \opdata{<title>}{2}{3}{4}{<doc>} forms:
#       a) <title> and <doc> get transformed as described below.
#       b) arguments 2,3,4 remain unchanged
#
#   2)  All \spad{} forms are transformed as described below.
#
#   3)  \xxdata{} forms are skipped over. They must be on a single line.
#
# Syntax of <title>
#       <title>   :: <form> [\optand <form>]* [\opand <form>]
#          The \optand and \opand are used for vertically stacking titles.
#           The entire title is wrapped with {\em ..}.
#       <form>    :: <name> ( [<arglist>] [<optPart>] )
#          Every form can have at most one \opt{} part for optional arguments.
#       <arglist> :: <name> <argtail>
#          A form always has open/close prens but arguments are optional.
#       <argtail> :: [, <name>]*
#          Arguments are separated by commas
#       <optPart> :: \opt{ <optBody> }
#          The optional part always has a body
#       <optBody> :: <arglist> | <argtail>
#          There are two forms \opt{a} when the first argument is optional,
#          and \opt{,a,b} when a later argument is optional
# Translations:
#          <name>   -->  {\tt <name>}
#          ( , )    -->  {\tt (}    {\tt ,}    {\tt )}
#          \opt{x}  -->  \lbrack translationOf(x) \rbrack
#
# Translations of \spad{x}:
#   Entire \spad{x} is formatted in math mode with translations as follows:
#   If x has the form <name>(<arglist>) [\$ <single-letter>],
#   where <name> is alphabetic with 3 or more alphabetic characters,
#   and where <arglist> can have an \optinner{} term with similar
#   syntax to that of \opt{} in <title>.
#          <name>   --> {\tt <name>}
#          ( , )    --> {\tt (}    {\tt ,}    {\tt )}
#          All other \spad{x} forms are formatted as $x$.

#-----------------------------------------------------------------------------
#                        MAIN PROGRAM
#-----------------------------------------------------------------------------
/^\\opdata/ {
        printf "%s","%------------------------------------------------------\n"
        printf "%s","\\opdata{"            # print \opdata{
        $0 = substr($0,9)                  # advance past \opdata{
        header()                           # print and process first argument
        k = index($0,"\}")
        $0 = substr($0,k+1)                # advance past }
        printf "%s","\}"                   # print matching }
        $0 =findSpad($0)
        next
}
/^\\keydata/ {
        $0 =findSpad($0)
        next
}
/^\\xxdata/ {                              # skip over \xxdata{} lines
        next
}
# otherwise
{
        $0 = findSpad($0)                  # transform \spad{} in other lines
        next
}
#-------------------------------------------------------------------------
#                        TRANSLATE <title>
#-------------------------------------------------------------------------
function header() {
        printf "%s","\{"                         # wrap braces around whole argument
        $0 = headform($0)
        k = index($0,"\\optand")
        while (k != 0) {
           printf "%s",substr($0,1,k+6)
           $0 = substr($0,k+7)
           $0 = headform($0)
           k = index($0,"\\optand")
        }
        k = index($0,"\\opand")
        while (k != 0) {
           printf "%s",substr($0,1,k+5)
           $0 = substr($0,k+6)
           $0 = headform($0)
           k = index($0,"\\optand")
        }
        printf "%s","\}"                          # wrap closing brace
}
#-------------------------------------------------------------------------
#                        TRANSLATE <form>
#-------------------------------------------------------------------------
function headform(form,j,k) {
        form = dropLeadingBlanks(form)
        k = index(form,"(")
        opname = substr(form,1,k-1)            # name not including left pren
        printf "%s%s%s","\{\{\\bf ",opname,"\}\\opLeftPren\{\}\}"
        form = substr(form,k+1)                # expose first argument
        form = dropLeadingBlanks(form)
        j = index(form,")")                    # check for empty argument
        if (j != 1) {
          c = getCommaIndex(form)              # try reading a comma
          k = c
          if (c == 0) k = getDelimiterIndex(form)# if none, read a right pren
          firstArg = substr(form,1,k-1)
          printf "%s",firstArg
          form = substr(form,k)                # expose comma after 1st arg
          form = commas(form)
        }
#-----------------------------------------------------------------------------
#                         TRANSLATE \opt{}
#--------------------------------------------------------------------------
        k = index(form,"\\opt{")
        if (k != 0) {
           printf "%s%s",substr(form,1,k-1),"\\bigLeftBrack "
           form = substr(form,k+5)
           k = closingBraceIndex(form)
           optform = substr(form,1,k-1)
           c = getCommaIndex(optform)              # try reading a comma
           if (c==0) {firstArg=optform} else {firstArg=substr(optform,1,c-1)}
           printf "%s",firstArg
           optform=substr(optform,c)
           optform = commas(optform)
           form = optform substr(form,k+1)
           printf "%s","\\bigRightBrack "
        }
        k =index(form,"\)")                # advance past right pren
        form = substr(form,k+1)
        printf "%s","\\opRightPren\{\}"
        return form
}
function commas(form,j,k) {
        while (substr(form,1,1) == ",") {
          form = substr(form,2)                # expose next argument
          k = getCommaIndex(form)
          # If no comma, go ONLY as far as a delimiter---remember you are
          # working the entire rest of the line here
          # Of course: stop earlier if you have an "\opt{}"
          if (k == 0) {
            j = index(form,"\\opt{")
            if (j != 0) {k=j} else {k = getDelimiterIndex(form)}
          }
          if (k==0) {
            arg=form
            form= ""
          } else {
            arg=substr(form,1,k-1)
            form=substr(form,k)
          }
          printf "%s%s","\{\\tt ,\}",arg
        }
        return form
}
#-----------------------------------------------------------------------------
#               SEARCH FOR \spad{} WITHIN A STRING
#-----------------------------------------------------------------------------
function findSpad(line,k) {
        original=line
        k = index(line,"\\spad\{")           # scan for \spad{}
        if (k == 0) {return finish(original)}# if not found, print/return
        prefix=substr(line,1,k-1)
        line = substr(line,k+6)              # expose name(...) inside \spad{}
        k = closingBraceIndex(line)
        if (k==0) {return finish(original)}
        printf "%s%s",prefix,"\\spad\{"
        form = substr(line,1,k-1)            # get form inside \spad{}
        line = substr(line,k+1)
        spadTran(form)
        printf "%s","\}"                     # print closing brace for \spad
        return findSpad(line)
}
function finish(line) {                      # print out a line
          printf "%s%s",line,"\n"
          return
}
#-----------------------------------------------------------------------------
#                 Transform string inside \spad{}
#-----------------------------------------------------------------------------

function spadTran(form,k) {
#  printf "%s%s%s","\n========",form,"------\n"
        k = index(form,"(")
        form=dropTrailingBlanks(form)        # kill trailing blanks
        hasRightPren= (substr(form,length(form),1) == "\)")
        hasDollar  = (substr(form,length(form)-1,1) == "\$")
        looksOk = (hasRightPren || hasDollar)
        if (k == 0 || !looksOk) {
           printf "%s",form                # no "(" so quit
           return
        }
        name = substr(form,1,k-1)
        m = length(name)
        kind = isAlphabetic(name)
        if (m < 3 || isAlphabetic(name) == "false") {
          printf "%s",form
          return
        }       # kill matching close pren
        if (hasRightPren) form=substr(form,1,length(form)-1)

        printf "%s%s%s","\{\{\\bf ",name,"\}\\opLeftPren\{\}\}"
        form = substr(form,k+1)      # expose first argument
        k = getCommaIndex(form)      # is there a comma?
        if (k==0) k = getDelimiterIndex(form) # if not, a delimiter
        firstArg = substr(form,1,k-1)
        printf "%s",firstArg
        form = substr(form,k)
        form = spadCommas(form)
        #-------------------- \optinner handler --------------------
        #   here, form may look like "arg\optinner{, args}
        #-----------------------------------------------------------
        k = index(form,"\\optinner{")
        if (k != 0) {
           printf "%s%s",substr(form,1,k-1),"\\bigLeftBrack "
           form = substr(form,k+10)
           k = closingBraceIndex(form)
           optform = substr(form,1,k-1)
           optform = spadCommas(optform)
           form = optform substr(form,k+1)
           printf "%s","\\bigRightBrack "
         }
         #form should now be JUST a right pren
         k = index(form,"\)")
         form=substr(form,k+1)
         printf "%s%s",form,"\\opRightPren\{\}"
}
function spadCommas(form,k) {
        form = dropLeadingBlanks(form)
        while (substr(form,1,1) == ",") {
          form = substr(form,2)     # expose next argument
          k = getCommaIndex(form)  # get index of delimiter (comma/right pren)
          if (k==0) {
            k = getDelimiterIndex(form)
            j = index(form,"\\optinner")
            if (j != 0 && j < k) k=j
          }
          if (k==0) {
            arg = form
            form=""
          } else {
            arg = substr(form,1,k-1)
            form=substr(form,k)
          }
          printf "%s%s","\{\\tt ,\}",arg
        }
        return form
}
#-----------------------------------------------------------------------------
#                   SEARCH FUNCTIONS
#-----------------------------------------------------------------------------
function getCommaIndex(line,i, j) {    # Returns index of ","
        char = substr(line,1,1)
        count=1
        level=0
        while (char != ",") {
          if (char == "\{")  {                   # move past {}
            while (level > 1 || char != "\}") {
              if (char == "\{") level += 1
              if (char == "\}") level -= 1
              count+=1
              line = substr(line,2)
              char = substr(line,1,1)
              if (length(line) == 0) {
                return 0
              }}
             level=0
            }
          else if (char == "\(")   {             # move past ()
            while (level > 1 || char != "\}") {
              if (char == "(") level += 1
              if (char == ")") level -= 1
              count+=1
              line = substr(line,2)
              char = substr(line,1,1)
              if (length(line) == 0) {
                return 0
              }}
            level=0
            line=substr(line,2)                 # move past closing pren
            count+=1
            }
          else if (char ~ /[\}\)\\]/) {
            return 0
          }
          count += 1
          line = substr(line,2)
          if (length(line)==0) return 0
          char = substr(line,1,1)
        }
        j = index(string,"\\op")
        if (j != 0 && j < count) count=j
        return count
}
function closingBraceIndex(line,level,char,count) {
        char = substr(line,1,1)
        level=1
        count=1
        while (level > 1 || char != "\}") {
          if (char == "\{") level += 1
          if (char == "\}") level -= 1
          line = substr(line,2)
          if (length(line) == 0) return 0
          char = substr(line,1,1)
          count += 1
        }
        return count
}
function getDelimiterIndex(line,i) {
#   getDelimiterIndex returns index of "{}()"
        char = substr(line,1,1)
        count=1
        level=0
        while (char !~ /[\}\)]/) {
          if (char == "\{")  {                   # move past {}
            while (level > 1 || char != "\}") {
              if (char == "\{") level += 1
              if (char == "\}") level -= 1
              count+=1
              line = substr(line,2)
              char = substr(line,1,1)
              if (length(line) == 0) {
                return 0
              }}
             level=0
            }
          else if (char == "\(")   {             # move past ()
            while (level > 1 || char != "\}") {
              if (char == "(") level += 1
              if (char == ")") level -= 1
              count+=1
              line = substr(line,2)
              char = substr(line,1,1)
              if (length(line) == 0) {
                return 0
              }}
            level=0
            }
          else if (char ~ /[\}\)\\]/) {
            return 0
          }
          count += 1
          line = substr(line,2)
          if (length(line)==0) return 0
          char = substr(line,1,1)
        }
        j = index(string,"\\op")
        if (j != 0 && j < count) count=j
        return count
}
#-----------------------------------------------------------------------------
#                   UTILITY FUNCTIONS
#-----------------------------------------------------------------------------

function isAlphabetic(name,i) {
        for (i = 1; i <= length(name); i+=1) {
          char = substr(name,1,1)
          if (char  !~ /^[A-Za-z][A-Za-z0-9]*/) return "false"
          name = substr(name,2)
        }
        return "true"
}
function min(m,n) {
        return m < n ? m : n
}
function dropLeadingBlanks(line) {
        while (substr(line,1,1) == " ") {
           printf "%s"," "
           line = substr(line,2)
        }
        return line
}
function dropTrailingBlanks(line) {
        while (substr(line,length(line),1) == " ") {
           line = substr(line,1,length(line)-1)
        }
        return line
}
