# Copyright The Numerical Algorithms Group Limited 1992-94.
#
# This awk script processes an .ht file or .htex and creates an AXIOM 
# test .input file on standard output. One argument is expected, which
# should be the full file name.

BEGIN   {
        printf("-- Test .input file generated by tinput.awk for %s\n\n", FILENAME)
}

/^\\begin{spadsrc}/ {
        i = 0
        getline
        while ($1 != "\\end{spadsrc}") {
          gsub(/\\/,"")
          print $0
          getline
        }
        next
}

/^\\begin{inputonly}/ {
        getline
        while (substr($1,1,15) != "\\end{inputonly}") {
                print $0
                getline
        }
        next
}

# delete lines between \begin{texonly} and \end{texonly}
/^\\begin{texonly}/     {
        while (substr($1,1,13) != "\\end{texonly}") # might have trailing %
                getline
        next
}

# delete lines between \texonly and \endtexonly
/^\\texonly/     {
        while (substr($1,1,11) != "\\endtexonly") # might have trailing %
                getline
        next
}

# handle discard stuff (delete the lines)
/^\\begin{discard}/   {
        while (substr($1,1,13) != "\\end{discard}") # might have trailing %
                getline
        next
}

/^\\spadpaste/ || /^\\spadcommand/ || /^\\graphpaste/ || /^\\spadgraph/ {
        # fix special characters
        gsub(/\\_/,"_")
        gsub(/\\\]/,"]")
        gsub(/\\\[/,"[")
        gsub(/\\}/,"}")
        gsub(/\\{/,"{")
        gsub(/\\#/,"#")
        gsub(/\\%/,"%")
        gsub(/\\\$/,"\$")
        gsub(/\\free{.*}/,"")
        gsub(/\\bound{.*}/,"")
        gsub(/\\ /," ")
# gsub with & just does not seem to work, even with \& in substitution string
#       gsub(/\\&/,"\&")
        pa = index($0,"\\&")
        while (pa > 0) {
          $0 = substr($0,1,pa-1) "&" substr($0,pa+2)
          pa = index($0,"\\&")
        }
        p = index($0,"{")
        # get rid of prefix and suffix
        $0 = substr($0,p+1)     # prefix
        # remove trailing blanks and last brace
        while (substr($0,length($0),1) == " ")
                $0 = substr($0,1,length($0)-1)
        if (substr($0,length($0),1) == "}")
                $0 = substr($0,1,length($0)-1)
        print $0
        next
}

/^\\head{/ {
        page = extractArg( $0, 3)
        printf("\n-- Input generated from %s\n)clear all\n\n", page)
        next
}

/^\\begin{page}/ {
        page = extractArg( $0, 2)
        printf("\n-- Input generated from %s\n)clear all\n\n", page)
        next
}

# otherwise
                {
        next
}

function endMacroIndex(line,parms,    pp,x,bc,cc) {
# assumes start of line is a macro call and returns position of final "}"
        x = 0
        pp = index(line,"{")
        if (pp != 0) {
          bc = 1
          for (x = pp+1; ; x++) {
            cc = substr(line,x,1)
            if (cc == "{")
              bc++
            else if (cc == "}") {
              bc--
              if (bc == 0) {
                parms--
                if (parms == 0)
                  break
              }
            }
          }
        }
        return x
}

function extractArg(line,num,   p,arg) {
# assumes line is a macro call and extracts the num-th arg
        arg = ""
        p = index(line,"{")
        if (p != 0) {
          line = substr(line,p)

          if (num > 1) {
            p = endMacroIndex(line,num-1)
            line = substr(line,p+1)
          }
          p = endMacroIndex(line,1)
          arg = substr(line,2,p-2)
        }
        return arg
}
